@@ -18,13 +18,9 @@ laws and state trade secret laws, punishable by civil and criminal penalties.
 
 name = "seqslab"
 
-__all__ = [
-
-]
+__all__ = []
 
 
-__version__ = "3.2.2"
+__version__ = "3.3.2.post0"
 
-LOGGING = {
-    "DIR_PATH": "/var/log/seqslab"
-}
\ No newline at end of file
+LOGGING = {"DIR_PATH": "/var/log/seqslab"}
