@@ -16,23 +16,38 @@ copies by sale, rental, lease or lending are violations of federal copyright
 laws and state trade secret laws, punishable by civil and criminal penalties.
 """
 
+# Standard Library
+import signal
 import sys
 
 from nubia import Nubia, Options
-
-from seqslab import auth, drs, wes, trs, workspace, user, role, organization
+from seqslab import auth, drs, organization, role, scr, trs, user, wes, workspace
 from seqslab.plugin import SQLBPlugin
 
 
+def signal_handler(sig, frame):
+    msg = "\nCtrl-c was pressed. Do you really want to exit? [y/n] "
+    sys.stderr.write(msg)
+    ans = input()
+
+    if ans == "y":
+        print("")
+        exit(1)
+    else:
+        print("", end="\r", flush=True)
+        print(" " * len(msg), end="", flush=True)  # clear the printed line
+        print("    ", end="\r", flush=True)
+
+
 def main():
+    signal.signal(signal.SIGINT, signal_handler)
     plugin = SQLBPlugin()
     shell = Nubia(
         name="seqslab-cli",
-        command_pkgs=[auth, drs, wes, trs, workspace, user, role, organization],
+        command_pkgs=[auth, drs, wes, trs, workspace, user, role, organization, scr],
         plugin=plugin,
         options=Options(
-            persistent_history=False,
-            auto_execute_single_suggestions=False
+            persistent_history=False, auto_execute_single_suggestions=False
         ),
     )
     sys.exit(shell.run())
