@@ -10,7 +10,4 @@ __version__ = "v3"
 PRIVATE_NAME = env.str("PRIVATE_NAME", "api")
 API_HOSTNAME = f"{PRIVATE_NAME}.seqslab.net"
 
-__all__ = [
-    API_HOSTNAME,
-    __version__
-]
+__all__ = [API_HOSTNAME, __version__]
