@@ -1,10 +1,10 @@
+# Standard Library
 import sys
 from typing import TypeVar
 
 from nubia import context
-from seqslab.trs.resource.base import BaseResource
 from seqslab.settings import TRS_RESOURCE_BACKEND
-
+from seqslab.trs.resource.base import BaseResource
 
 """
 Copyright (C) 2022, Atgenomix Incorporated.
@@ -29,14 +29,14 @@ class Factory:
     """Storage backend factory"""
 
     @staticmethod
-    def load_resource(workspace: str) -> Resource:
+    def load_resource() -> Resource:
         backend = context.get_context().args.backend
         name = TRS_RESOURCE_BACKEND.get(backend)
         mod, mem = name.rsplit(".", 1)
         __import__(mod)
         module = sys.modules[mod]
         backend_class = getattr(module, mem)
-        return backend_class(workspace)
+        return backend_class()
 
 
 _factory = Factory()
@@ -44,4 +44,3 @@ _factory = Factory()
 
 def trs_resource():
     return _factory
-
