@@ -5,9 +5,7 @@ environ.Env.read_env("/etc/seqslab/cli_apps.env")
 
 name = "users"
 
-__all__ = [
-
-]
+__all__ = []
 
 __version__ = "v3"
 
